package com.foresee.KnowledgeBase.SynchronizeData.zsdy.util;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Base64;
import java.util.zip.Deflater;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;

import com.google.gson.JsonObject;

public class HmacSignUtil {
	public static final String SDKAPPID = "1400386304";
    public static final String SECRETKEY = "917cabe380e071627e3e6700df2b4ee042d46d944b06d26f949c0a9539524367";
    public static final long TIMES = System.currentTimeMillis();

    public static String genTestUserSig(long times) {
        return genTLSSignature(SDKAPPID, times, SECRETKEY);
    }

    /**
     * 解密算法     
     * @param sdkappid appKey     
     * @param times ts     
     * @param priKeyContent appSecret     
     * @return String
     */
    public static String genTLSSignature(String sdkappid, long times, String priKeyContent) {
        if (StringUtils.isBlank(priKeyContent)) {
            return "";
        }
        String identifier = DigestUtils.md5Hex(sdkappid);
        JsonObject sigDoc = new JsonObject();

        sigDoc.addProperty("TLS.ver", "2.0");
        sigDoc.addProperty("TLS.sdkappid", sdkappid);
        sigDoc.addProperty("TLS.identifier", identifier);

        String sig = hmacsha256(sdkappid, identifier, times, priKeyContent);
        if (sig.length() == 0) {
            return "";
        }
        sigDoc.addProperty("TLS.sig", sig);
        sigDoc.addProperty("TLS.time", times);
        Deflater compressor = new Deflater();
        compressor.setInput(sigDoc.toString().getBytes(Charset.forName("UTF-8")));
        compressor.finish();
        byte[] compressedBytes = new byte[2048];
        int compressedBytesLength = compressor.deflate(compressedBytes);
        compressor.end();
        return new String(base64EncodeUrl(Arrays.copyOfRange(compressedBytes, 0, compressedBytesLength)));
    }

    /**
     * 加密算法
     * @param sdkappid appKey
     * @param currTime ts     
     * @param identifier identifier     
     * @param priKeyContent appSecret     
     * @return String
     */
    private static String hmacsha256(String sdkappid, String identifier, long currTime, String priKeyContent) {
        String contentToBeSigned = "TLS.identifier:" + identifier + "\n" + "TLS.sdkappid:" + sdkappid + "\n" + "TLS.time:" + currTime + "\n";
        try {
            byte[] byteKey = priKeyContent.getBytes("UTF-8");
            Mac hmac = Mac.getInstance("HmacSHA256");
            SecretKeySpec keySpec = new SecretKeySpec(byteKey, "HmacSHA256");
            hmac.init(keySpec);
            byte[] byteSig = hmac.doFinal(contentToBeSigned.getBytes("UTF-8"));
            return new String(Base64.getEncoder().encodeToString(byteSig));
        } catch (UnsupportedEncodingException e) {
            return "";
        } catch (NoSuchAlgorithmException e) {
            return "";
        } catch (InvalidKeyException e) {
            return "";
        }
    }

    private static byte[] base64EncodeUrl(byte[] input) {
        byte[] base64 = new String(Base64.getEncoder().encodeToString(input)).getBytes();
        for (int i = 0; i < base64.length; ++i) {
            switch (base64[i]) {
                case '+':
                    base64[i] = '*';
                    break;
                case '/':
                    base64[i] = '-';
                    break;
                case '=':
                    base64[i] = '_';
                    break;
                default:
                    break;
            }
        }
        return base64;
    }
}
